/*
 * Copyright (c) 1998-99 The WebApp Framework.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The WebApp Framework may be used for evaluation, private or
 * educational purposes without fee.
 *
 * 4. The right to redistribute The WebApp Framework or any part of it
 * as part of free software is hereby granted. Redistribution as part
 * of a commercial product is NOT permitted in any form without prior
 * written permission.
 *
 * 5. Every modification must be notified to The WebApp Framework, i.e.
 * Peter Rossbach and Hendrik Schreiber, and redistribution of the
 * modified code without prior notification is NOT permitted in any form.
 *
 * 6. All advertising materials mentioning features or use of this
 * software must display the following acknowledgment: "This product
 * includes software developed by Peter Rossbach and Hendrik Schreiber
 * (http://www.webapp.de/)."
 *
 * 7. The names "WebApp" and "WebApp Framework" must not be used to
 * endorse or promote products derived from this software without prior
 * written permission.
 *
 * 8. Redistributions of any form whatsoever must retain the following
 * acknowledgment: "This product includes software developed by Peter
 * Rossbach and Hendrik Schreiber (http://www.webapp.de/)."
 *
 * 9. As this release also contains software by Sun Microsystems the
 * following conditions have to be met, too. They apply to the
 * files lib/servlet.jar, lib/mail.jar and lib/activation.jar
 * contained in this release.
 *
 * 10. Java Platform Interface.  Licensee may not modify the Java Platform
 * Interface (JPI, identified as classes contained within the javax
 * package or any subpackages of the javax package), by creating additional
 * classes within the JPI or otherwise causing the addition to or modification
 * of the classes in the JPI.  In the event that Licensee creates any 
 * Java-related API and distribute such API to others for applet or
 * application development, you must promptly publish broadly, an accurate 
 * specification for such API for free use by all developers of Java-based 
 * software.  
 *
 * 11. Restrictions.  Software is confidential copyrighted information of Sun and 
 * title to all copies is retained by Sun and/or its licensors.  Licensee 
 * shall not modify, decompile, disassemble, decrypt, extract, or otherwise 
 * reverse engineer Software.  Software may not be leased, assigned, or 
 * sublicensed, in whole or in part.  Software is not designed or intended 
 * for use in on-line control of aircraft, air traffic, aircraft navigation 
 * or aircraft communications; or in the design, construction, operation or 
 * maintenance of any nuclear facility.  Licensee warrants that it will not 
 * use or redistribute the Software for such purposes.
 *
 * 12. Disclaimer of Warranty.  Software is provided "AS IS," without a warranty 
 * of any kind. *  ALL EXPRESS OR IMPLIED REPRESENTATIONS AND WARRANTIES, 
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED.  
 *
 * 13. Termination.  This License is effective until terminated.  Licensee may 
 * terminate this License at any time by destroying all copies of Software.  
 * This License will terminate immediately without notice from Sun if Licensee
 * fails to comply with any provision of this License.  Upon such termination,
 * Licensee must destroy all copies of Software.
 *
 * 14. Export Regulations.  Software, including technical data, is subject to U.S.
 * export control laws, including the U.S.  Export Administration Act and its
 * associated regulations, and may be subject to export or import regulations
 * in other countries.  Licensee agrees to comply strictly with all such 
 * regulations and acknowledges that it has the responsibility to obtain
 * licenses to export, re-export, or import Software.  Software may not be 
 * downloaded, or otherwise exported or re-exported (i) into, or to a national
 * or resident of, Cuba, Iraq, Iran, North Korea, Libya, Sudan, Syria or any 
 * country to which the U.S. has embargoed goods; or (ii) to anyone on the 
 * U.S. Treasury Department's list of Specially Designated Nations or the U.S.
 * Commerce Department's Table of Denial Orders.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER ROSSBACH AND HENDRIK SCHREIBER
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PETER
 * ROSSBACH AND HENDRIK SCHREIBER BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For more information on The WebApp Framework, Peter Rossbach or
 * Hendrik Schreiber, please see <http://www.webapp.de/>.
 *
 */

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Server, which is capable of complying a
 * single <code>GET</code> request.
 *
 * @author Hendrik Schreiber, Peter Rossbach
 * @version $Id: OneShotHttpd.java,v 1.7 2000/07/15 12:23:39 Hendrik Exp $
 * @see SimpleHttpd
 * @see SimpleHttpd2
 */
public class OneShotHttpd 
{
	/**
	 * Source version
	 */
	public static String vcid = "$Id: OneShotHttpd.java,v 1.7 2000/07/15 12:23:39 Hendrik Exp $";

	/**
	 * Document root
	 */
	protected static File docRoot;

	/**
	 * The port the server will listen to
	 */
	public final static int HTTP_PORT = 8080;

	/**
	 * Waits for an <code>GET</code> request and performs it.
	 * Objects are searched for relatively to the current directory
	 * (Document root = "./"). If no file is specified, only
	 * a directory, the file <code>index.html</code> is delivered.<br>
	 * If the request is not a <code>GET</code> request, the error
	 * message 400, <code>Bad Request</code>, is sent to the client.
	 */
	public static void main(String argv[])
	{
		try
		{
			docRoot = new File(".");
			ServerSocket listen = new ServerSocket(HTTP_PORT);
			Socket client = listen.accept();
			BufferedReader is = new BufferedReader(new InputStreamReader(client.getInputStream()));
			DataOutputStream os = new DataOutputStream(client.getOutputStream());
			String request = is.readLine();
			System.out.println("Request: " + request);
			StringTokenizer st = new StringTokenizer(request);
			if((st.countTokens() == 3) && st.nextToken().equals("GET"))
			{
				String filename = docRoot.getPath() + st.nextToken();
				if(filename.endsWith("/") || filename.equals(""))
					filename += "index.html";
				File file = new File(filename);
				sendDocument(os,file);
			}
			else
				System.err.println("400 Bad Request");
			is.close();
			os.close();
			client.close();
		}
		catch(IOException ioe)
		{
			System.err.println("Error: " + ioe.toString());
		}
	}

	/**
	 * Reads the file, specified in <code>request</code> and writes it to
	 * the OutputStream.<br>
	 * If the file could not be found, the error message 404,
	 * <code>Not Found</code>, is returned.
	 *
	 * @exception IOException in case writing to the <code>BufferedOutputStream</code>
	 *    fails.
	 */
	public static void sendDocument(DataOutputStream out, File file) throws IOException
	{
		try
		{
			BufferedInputStream in = new BufferedInputStream(new FileInputStream(file));
			byte[] buf = new byte[1024];
			int len;
			while((len = in.read(buf,0,1024)) != -1)
			{
				out.write(buf,0,len);
			}
			in.close();
		}
		catch(FileNotFoundException fnfe)
		{
			System.err.println("404 Not Found");
		}
	}

}

// end of class